# -*- coding: utf-8 -*-

import os
from twisted.python import threadpool
from twisted.web import server, resource, wsgi, static
from twisted.internet import reactor
from twisted.internet.endpoints import serverFromString
from django.core.handlers.wsgi import WSGIHandler
from django.core.management import call_command
from xost import __version__

server.version = 'Xost/%s' % __import__('xost').__version__

class WSGI(resource.Resource):
    def __init__(self, min_threads=5, max_threads=20):
        resource.Resource.__init__(self)
        self.pool = threadpool.ThreadPool(min_threads, max_threads)
        reactor.addSystemEventTrigger('after', 'shutdown', self.pool.stop)
        self.wsgi_resource = wsgi.WSGIResource(reactor, self.pool, WSGIHandler())
        self.pool.start()

    def getChild(self, path, request):
        path0 = request.prepath.pop(0)
        request.postpath.insert(0, path0)
        return self.wsgi_resource

    pass


class File(static.File):
    def __init__(self, path, defaultType="text/html", ignoredExts=(), registry=None, allowExt=0, dirList=False):
        self.dirList = dirList
        super(File, self).__init__(path, defaultType, ignoredExts, registry, allowExt)

    def directoryListing(self):
        if not self.dirList:
            return resource.ForbiddenResource()
        return super(File, self).directoryListing()

    pass


class Site(server.Site):
    def __init__(self, config):
        self.config = config
        self.root = WSGI(config.min_threads, config.max_threads)
        self.wsgi_pool = self.root.pool
        self.root.putChild('static', File(config.static_root))
        self.root.putChild('media', File(config.media_root))
        server.Site.__init__(self, self.root, config.log_path)

    pass


class Server(object):
    version = server.version

    def __init__(self, config):
        self.config = config

    def run(self):
        conf = self.config
        if conf.debug:
            print server.version
            print
            print 'Resources:'
            print '\troot:', conf.root
            print '\tstatic:', conf.static_root
            print '\tmedia:', conf.media_root
            print '\tlog:', conf.log_path
        if conf.log_path and not os.path.exists(conf.log_path):
            if not os.path.exists(os.path.dirname(conf.log_path)):
                os.makedirs(os.path.dirname(conf.log_path))
            f = open(conf.log_path, 'w')
            f.close()
        if conf.collect_static:
            call_command('collectstatic', interactive=False)
        site = Site(conf)
        try:
            if conf.address:
                srv = serverFromString(reactor, conf.address)
                srv.listen(site)
            else:
                reactor.listenTCP(conf.port, site)
            reactor.run()
        except Exception, ex:
            if site.root:
                site.wsgi_pool.stop()
            raise ex
        pass

    pass



# -*- coding: utf-8 -*-


from django.utils.importlib import import_module

import os

class Config:
    address = None
    port = 8080
    min_threads = 5
    max_threads = 20
    root = None
    static_root = None
    media_root = None
    log_path = None
    collect_static = True
    debug = False

    def __init__(self):
        self.root = self.root or os.getcwd()
        self.static_root = self.static_root or os.path.join(self.root, 'static')
        self.media_root = self.media_root or os.path.join(self.root, 'media')
        if self.address:
            self.address = self.address.strip()
            if self.address.find(':lockfile=') == -1:
                self.address += ':lockfile=0'
            if self.address.find(':mode=') == -1:
                self.address += ':mode=0660'
            pass

    pass


class EmbdedConfig(Config):
    def __init__(self, settings=None):
        settings = settings or import_module(os.environ['DJANGO_SETTINGS_MODULE'])
        if hasattr(settings,'XOST'):
            for key, val in getattr(settings, 'XOST').iteritems():
                setattr(self, key, val)
            pass
        Config.__init__(self)

    pass




# -*- coding: utf-8 -*-

import os
from optparse import make_option
from django.core.management import BaseCommand
from xost.config import EmbdedConfig
from xost.server import Server


class Command(BaseCommand):
    help = '%s embded server' % Server.version
    can_import_settings = True

    option_list = BaseCommand.option_list + (
        make_option('-d', '--daemonize',
            action='store_true',
            dest='daemonize',
            help='launch as daemon (whether to detach from terminal)'
        ),
        make_option('--pid',
            action='store',
            dest='pidfile',
            help='write the spawned process-id to this file. '
        ),
        make_option('-a', '--address',
            action='store',
            dest='address',
            help='string configuration listen addresses'
        ),
        make_option('-p', '--port',
            action='store',
            dest='port',
            type='int',
            help='a port number on which to listen'
        ),
        make_option('--min_threads',
            action='store',
            dest='min_threads',
            type='int',
            help='minimum number of threads in the pool'
        ),
        make_option('--max_threads',
            action='store',
            dest='max_threads',
            type='int',
            help='maximum number of threads in the pool'
        ),
        make_option('--root',
            action='store',
            dest='root',
            help='path to working (root) folder'
        ),
        make_option('--static_root',
            action='store',
            dest='static_root',
            help='path to static file folder'
        ),
        make_option('--media_root',
            action='store',
            dest='static_root',
            help='path to media file folder'
        ),
        make_option('--log_path',
            action='store',
            dest='log_path',
            help='path to log folder'
        ),
        make_option('--no_static',
            action='store_false',
            dest='collect_static',
            help='do not collect static files to  static root folder before launch'
        ),
        )

    def handle(self, *args, **options):
        from django.conf import settings

        conf = EmbdedConfig(settings)
        settings.STATIC_ROOT = conf.static_root
        settings.MEDIA_ROOT = conf.media_root

        if not conf.debug:
            conf.debug = settings.DEBUG

        for key, val in options.iteritems():
            if hasattr(conf, key) and val is not None:
                setattr(conf, key, val)
            pass

        #if not conf.log_path:
        #    log_name = os.environ['DJANGO_SETTINGS_MODULE']
        #    log_name = log_name[:log_name.rfind('.')] + '.access.log'
        #    conf.log_path = os.path.join(conf.root, 'log', log_name)


        server = Server(conf)

        if options['daemonize']:
            from django.utils.daemonize import become_daemon
            become_daemon(our_home_dir=conf.root)

        if options["pidfile"]:
            fp = open(options["pidfile"], "w")
            fp.write("%d\n" % os.getpid())
            fp.close()

        server.run()

    pass



#!/usr/bin/env python

from distutils.core import setup


setup(
    name='xost',
    version=__import__('xost').__version__,
    packages=['xost', 'xost.management', 'xost.management.commands'],
    url='',
    license='MIT',
    author='gratromv',
    #author_email='',
    description='Xost is a small embedded server which simplifies hosting of django applications',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python',
    ],
    install_requires=['django', 'Twisted',],
)
